#region using
using System;
using System.Collections;
using System.Collections.Specialized;
using System.Reflection;
using System.Xml.Serialization;

using GBPVR.Public;

using RecorderHelper;

using DShowNET;
#endregion

namespace SoftwareRecorder
{
	/// <summary>
	/// Summary description for SoftwareRecorderConfig.
	/// </summary>
	[XmlType("captureSource")]
	public class SoftwareRecorderConfig : IRecordingSourceConfiguration
	{

		#region Constructor / Destructor

		public SoftwareRecorderConfig()
		{
			// constructor
			string version = Assembly.GetExecutingAssembly().GetName().Version.ToString();
			Logger.Verbose("SoftwareRecorderConfig[" + this.GetHashCode() + "]: SoftwareRecorderConfig(version=" + version + ")");
		}

		public SoftwareRecorderConfig(SoftwareRecorder recordingSource)
		{
			// constructor
			string version = Assembly.GetExecutingAssembly().GetName().Version.ToString();
			Logger.Verbose("SoftwareRecorderConfig[" + this.GetHashCode() + "]: SoftwareRecorderConfig(version=" + version + ", recordingSource=[" + recordingSource.GetHashCode() + "])");
			this.recordingSource = recordingSource;
		}

		#endregion

		#region Internal Variables

		/// <summary> captureSourceOID for device. </summary>
		[XmlAttribute("oid"),XmlHashKey()]
		internal int					captureSourceOID;

		/// <summary> the unique identifier of the video capture device. </summary>
		internal string					uniqueVideoDeviceIdentifier;

		/// <summary> the capture pin on the video capture device. </summary>
		internal string					uniqueVideoDeviceCapture;

		/// <summary> the category of the capture pin on the video capture device. </summary>
		internal Guid					uniqueVideoDeviceCaptureCategory = PinCategory.Capture;

		/// <summary> the preview pin on the video capture device. </summary>
		internal string					uniqueVideoDevicePreview;

		/// <summary> the unique identifier of the audio capture device. </summary>
		internal string					uniqueAudioDeviceIdentifier;

		/// <summary> the source of audio on the audio capture device. </summary>
		internal string					uniqueAudioDeviceSource;

		/// <summary> the capture pin on the audio capture device. </summary>
		internal string					uniqueAudioDeviceCapture;

		/// <summary> the category of the capture pin on the audio capture device. </summary>
		internal Guid					uniqueAudioDeviceCaptureCategory = PinCategory.Capture;

		/// <summary> the mixer pin on the audio capture device. </summary>
		internal string					uniqueAudioDeviceMixer;

		/// <summary> the preferred routings for the crossbar device. </summary>
		internal string					crossbarRoutings;

		/// <summary> the video standard for the tuner device. </summary>
		internal AnalogVideoStandard	videoStandard;

		/// <summary> the country code for the tuning space. </summary>
		internal int					countryCode = 1;

		/// <summary> the video standard for the tuner device. </summary>
		internal TunerInputType			tuningType;

		/// <summary> the fixed channel to tune to. </summary>
		internal int					fixedChannel;

		/// <summary> select this to use the video capture card as an audio source for the audio capture card. </summary>
		internal bool					videoIsAudioSource = false;

		/// <summary> select this to force audio to render in Preview Mode. </summary>
		internal bool					audioRendersInPreview = false;

		/// <summary> select this to force recording to always start twice. </summary>
		internal bool					startRecordTwice = false;

		/// <summary> select this to force tuning to always occur twice. </summary>
		internal bool					tuneTwice = false;

		/// <summary> the timeout for detecting stuck recordings (in seconds). </summary>
		internal int					stuckTimeout = 5;

		/// <summary> the unique identifier of the video encoder device. </summary>
		internal string					uniqueVideoEncoderIdentifier;

		/// <summary> the unique identifier of the audio encoder device. </summary>
		internal string					uniqueAudioEncoderIdentifier;

		/// <summary> the unique identifier of the audio/video muxer device. </summary>
		internal string					uniqueMuxerIdentifier;

		/// <summary> the operation to perform during a channel change. </summary>
		internal ChannelChangeOperation	channelChangeOperation = ChannelChangeOperation.Stop;

		/// <summary> the list of Quality settings for recording. </summary>
		[XmlArray(),XmlArrayItem(typeof(Quality))]
		internal Hashtable				qualityProfiles = new Hashtable();

		/// <summary> the recording source used for this configuration. </summary>
		[NonSerialized()]
		internal SoftwareRecorder		recordingSource;

		#endregion

		#region IRecordingSourceConfiguration Members

		public IRecordingSource getRecordingSource()
		{
			//Logger.Verbose("SoftwareRecorder[" + this.GetHashCode() + "]: getRecordingSource, return " + this.GetHashCode());
			if (recordingSource == null)
			{
				recordingSource = new SoftwareRecorder(this);
			}
			return recordingSource;
		}

		public string getName()
		{
			Logger.Verbose("SoftwareRecorderConfig[" + this.GetHashCode() + "]: getName, return WDM Software Encoder Recorder");
			return "WDM Software Encoder Recorder";
		}

		public void Delete(System.Data.OleDb.OleDbConnection connection, int captureSourceOID)
		{
			Logger.Verbose("SoftwareRecorderConfig[" + this.GetHashCode() + "]: Delete(" + captureSourceOID + ")");
			Config.getConfig().Delete(captureSourceOID);
		}

		public void Load(int captureSourceOID)
		{
			Logger.Verbose("SoftwareRecorderConfig[" + this.GetHashCode() + "]: Load(" + captureSourceOID + ")");
			this.captureSourceOID = captureSourceOID;
			Config.getConfig().Load(captureSourceOID, this);
			recordingSources[captureSourceOID] = this;
		}

		public string getConfigurationFormClassName()
		{
			Logger.Verbose("SoftwareRecorderConfig[" + this.GetHashCode() + "]: getConfigurationFormClassName, return " + typeof(RecorderConfig).FullName);
			return typeof(RecorderConfig).FullName;
		}

		public void Save(System.Data.OleDb.OleDbConnection connection, int captureSourceOID)
		{
			Logger.Verbose("SoftwareRecorderConfig[" + this.GetHashCode() + "]: Save(" + captureSourceOID + ")");
			Config.getConfig().Save(captureSourceOID, this);
		}

		#endregion

		#region Singleton Implementation

		/// <summary> the recording source used for this configuration. </summary>
		[NonSerialized()]
		private static HybridDictionary	recordingSources = new HybridDictionary();

		internal static SoftwareRecorderConfig getConfigFromMemory(int captureSourceOID)
		{
			SoftwareRecorderConfig config = (SoftwareRecorderConfig)recordingSources[captureSourceOID];
			Logger.Verbose("SoftwareRecorderConfig: getConfigFromMemory(" + captureSourceOID + "), return [" + (config != null ? config.GetHashCode().ToString() : "null") + "]");
			return config;
		}

		#endregion

	}
}
